package xmdv.dataproc;

/***********************************************************
**  CopyRight (c) by Xmdv Group, CS Department of WPI, 2003
**  All Rights Reserved.
**
**  Author: Jing Yang
**  Date:   06/2003
***********************************************************/

/**
 * Abstract class for managing a dimension, i.e., a variable (column) in a dataset.
 *
 * @author  Jing Yang
 * @version v1.0
 */
public abstract class AbstractDimension {
    // Variables declaration
    protected String name;
    protected int cardinality;
    protected String origMaxString = null;
    protected String origMinString = null;
    // End of variables declaration
    
    /** constructor */
    public AbstractDimension() {
    }
    
    /**
     * Sets the maximum value of this dimension.  
     * @param maxString the string that contains the maximum value
     */
    public abstract void setMax(String maxString);

    /**
     * Sets the minimum value of this dimension.  
     * @param minString the string that contains the minimum value
     */
    public abstract void setMin(String minString);

    /**
     * Gets the string that contains the maximum value passed in in the setMax method.  
     * @return the string that contains the maximum value passed in in the setMax method
     */
    public String getOrigMaxString() {
        return origMaxString;
    }
    
    /**
     * Gets the string that contains the minimum value passed in in the setMin method.  
     * @return the string that contains the minimum value passed in in the setMin method
     */
    public String getOrigMinString() {
        return origMinString;
    }
    
    /**
     * Sets the name of this dimension. 
     * @param name the name to be set
     */
    public void setName(String name) {
        this.name = name;
    }
    
    /**
     * Gets the name of this dimension. 
     * @return the name of this dimension
     */
    public String getName() {
        return name;
    }
    
    /**
     * Sets the cardinality of this dimension. 
     * If the cardinality to be set is less than 1, it will be set to 1  
     * @param cardinality the cardinality to be set
     */
    public void setCardinality(int cardinality) {
        if (cardinality < 1)
            cardinality = 1;
        this.cardinality = cardinality;
    }
    
    /**
     * Gets the cardinality of this dimension.  
     * @return the cardinality of this dimension
     */
    public int getCardinality() {
        return cardinality;
    }
    
    /**
     * Adds a rim to this dimension so that the real range of this 
     * dimension is a little bit larger that the input range  
     */
    public abstract void addRim();

    /**
     * Gets a message contains the dimension name and a data value.  
     * @param norValue the normalized value of the data
     */
    public abstract String getNamedValueMessage(double norValue);

    /**
     * Gets a message contains a data value.  
     * @param norValue the normalized value of the data
     */
    public abstract String getValueMessage(double norValue);

    /**
     * Gets the normalized value of a data.  
     * @param valueString the data in a string
     */
    public abstract double getNorValue(String valueString);
}
