package xmdv.graphics;



/***********************************************************

**  CopyRight (c) by Xmdv Group, CS Department of WPI, 2003

**  All Rights Reserved.

**

**  Author: Jing Yang

**  Date:   06/2003

***********************************************************/



import java.awt.geom.Point2D;

import java.lang.Math;

import xmdv.dataproc.*;



/**

 * This class is an implementation of the abstract class FlatDisplay using 

 * the Star Glyph technique.

 *

 * @author  Jing Yang

 * @version v1.0

 */

public class FlatGlyph extends FlatDisplay{

    private Point2D.Double prevPosition;

    private double[][] glyphBase;

    private int dataNum;

    private double blockSize;

    private int xBlockNum; 

    private Point2D.Double[] poly;

    private double[] angles;

    private double[] blockCosAngles;

    private double[] blockSinAngles;

    

    /* inits: pt = point, pts = points, lo = low, hi = high, num = number 

     *        nor = normalized, prev = previous, plotIndex = dimension 

     *        min = minimum, max = maximum */

        

    /** Creates a new instance of FlatGlyph display*/

    public FlatGlyph () {

    }

    

    public FlatGlyph(grCanvas canvas, ColorManager colorManager, Dataset dataset) {

        setCanvas(canvas);

        setColorManager(colorManager);

        setDataset(dataset);

    }

    

    protected boolean isDimensionalityValid() {

        if ( dataset.getCurrDimNum() < 1 ) {

            System.out.println("No sufficent dimensions for this kind of display.");

            return false;

        } else 

            return true;

    }

    

    protected void setGlyphBase() {

        double halfBlockSize = .5 * blockSize;

    

        for ( int i = 0; i < dataNum; i++ ) {

            glyphBase[0][i] = blockSize * (i % xBlockNum) + halfBlockSize;

            glyphBase[1][i] = 1 - blockSize * (i / xBlockNum) - halfBlockSize;

        }

    }

    

    protected void setParameters() {

        super.setParameters();

    

        int i;

    

        if (dimNum != 0) {

            angles = new double[dimNum];

            blockSinAngles = new double[dimNum];

            blockCosAngles = new double[dimNum];

            double angle = 2.0 * Math.PI / dimNum;

            for (i = 0; i < dimNum; i++) {

                angles[i] = angle * i;

            }

            poly = new Point2D.Double[dimNum + 1];

            for (i = 0; i < dimNum + 1; i++)

                poly[i] = new Point2D.Double();

        }

        

        dataNum = dataset.getDataNum();

        if (dataNum != 0 ) {

            blockSize = 1.0 / (1.0 + Math.sqrt(dataNum));

            xBlockNum = (int)( 0.5 + Math.sqrt(dataNum) );

            for (i = 0; i < dimNum; i++) {

                blockSinAngles[i] = Math.sin(angles[i]) * blockSize / 2;

                blockCosAngles[i] = Math.cos(angles[i]) * blockSize / 2;

            }

    

            glyphBase = new double[2][dataNum];

            setGlyphBase();    

        }            

    }

    

    

    protected void drawSingleData(double[] norData, int dataIndex) {

        Point2D.Double pt;

    

        Point2D.Double base;

        base = new Point2D.Double(glyphBase[0][dataIndex], glyphBase[1][dataIndex]); 

        

        for (int i = 0; i < dimNum; i++) {

            poly[i].setLocation(glyphBase[0][dataIndex] + norData[i] * blockCosAngles[i],  

                                glyphBase[1][dataIndex] + norData[i] * blockSinAngles[i]);

            canvas.drawLine(base, poly[i]);

        }

        

        poly[dimNum].setLocation(poly[0].getX(), poly[0].getY());

        canvas.drawLines(poly);

    }

    

    protected void drawSingleBrush(Brush brush) {

    }

    

    protected void drawAxes() {

    }

    

    protected void inputMouseMotion(Point2D.Double pt)

    {

        if (inputMode == CANVAS_PAN) {

            canvas.pan(prevPosition, pt);

            canvas.refresh();

        }

        else {

            int[] dataIndex= new int[1];

            if (! getDataIndex(pt, dataIndex))

                return;

            glyphCaption(dataIndex[0]);

        }    

    }

    

    protected void inputButtonPress(int button, Point2D.Double pt) {

        inputMode = NONE;

        prevPosition = pt;

    

        if ((!canvas.altkey) && (!canvas.controlkey) && button == 3) {

            inputMode = CANVAS_PAN;

            canvas.setMoveCursor();

            return;

        }  

    

        if (button == 1) {

            int[] dataIndex= new int[1];

            if (! getDataIndex(pt, dataIndex))

                return;

    

            if (canvas.shiftkey) {

                inputMode = MULTI_SELECT;

                inputGlyphMultiSelect(dataIndex[0]);

            }

            else {

                inputMode = TRANSLATE_BRUSH;

                inputGlyphMoveBrush(dataIndex[0]);

                dataset.brush();

                dataset.refreshAllCanvas();

            }

            return;

        } 

    }

    

    

    protected void inputButtonRelease(int button, Point2D.Double pt)

    {

        switch (inputMode) {

    

            case CANVAS_PAN:

                canvas.setDefaultCursor();

                break;

    

            default: break;

        }

    

        inputMode = NONE;

    }

    

    protected boolean getDataIndex(Point2D.Double pt, int[] dataIndex)

    {

        dataIndex[0] = (int)((1 - pt.getY()) / blockSize) * xBlockNum + (int)(pt.getX() / blockSize);

        if (dataIndex[0] >= dataNum)

            return false;

        else 

            return true;    

    }

    

    protected void inputGlyphMultiSelect(int dataIndex) {

        double[] norData = dataset.getCurrNorData(dataIndex);

    

        if (dataset.multiSelectContains(dataIndex)){

            dataset.multiSelectRemovePoint(dataIndex);

            setQuickDrawCase0(norData, dataIndex, colorManager.getColor("data"));

            canvas.paintImmediately(0, 0, canvas.getWidth(), canvas.getHeight()); 

        } else {

            dataset.multiSelectAddPoint(dataIndex);

            setQuickDrawCase0(norData, dataIndex, colorManager.getColor("paint"));

            canvas.paintImmediately(0, 0, canvas.getWidth(), canvas.getHeight()); 

        }

    }

    

    protected void inputGlyphMoveBrush(int dataIndex)

    {

        double[] norData = dataset.getCurrNorData(dataIndex);

    

        for (int i = 0; i < dimNum; i++) {

            brush.shift(i, norData[i] - brush.getNorCenter(i));

        }

    }

    

    protected void glyphCaption(int dataIndex) {

        double[] norData;

        norData = dataset.getCurrNorData(dataIndex);

        String valueMessage = "";

        for (int i = 0; i < dimNum; i++) {

            valueMessage += dims[i].getNamedValueMessage(norData[i]) + " ";

        }

        canvas.showMessage(valueMessage);

    }

}

